@@ -2,17 +2,16 @@
 #include <command.h>
 #include <asm/cache.h>
 #include "sys_dep.h"
-#include "vesa.h"
+#include "../../../drivers/bios_emulator/vesa.h"
 
 int do_vesa(cmd_tbl_t * cmdtp, int flag, int argc, char *argv[])
 {
-  DECLARE_GLOBAL_DATA_PTR;
-  	DoVesa(argc, argv);
-  return 0;
+	DoVesa(argc, argv);
+	return 0;
 }
 
 U_BOOT_CMD(
 	vesa,    5,     1,     do_vesa,
-	"vesa    - run a vesa bios call\n",
-	"mode"
+	"set a vesa mode\n",
+	"hex mode"
 );
