@@ -32,21 +32,29 @@
 
 GNU_FPOST_ATTR
 
-int fpu_post_test_math3 (void)
+int fpu_post_test_math1 (void)
 {
-	volatile long double dfrom = 1.1;
-	volatile long double m1;
-	volatile long double m2;
-	volatile unsigned long mant_long;
+	volatile double a, *p;
+	double c, d;
+	volatile double b;
 
-	m1 = dfrom / 2.0;
-	m2 = m1 * 4294967296.0;
-	mant_long = ((unsigned long) m2) & 0xffffffff;
+	d = 1.0;
+	p = &b;
 
-	if (mant_long != 0x8ccccccc) {
-		post_log ("Error in FPU math3 test\n");
+	do
+	{
+		c = d;
+		d = c * 0.5;
+		b = 1 + d;
+	} while (b != 1.0);
+
+	a = 1.0 + c;
+
+	if (a == 1.0) {
+		post_log ("Error in FPU math1 test\n");
 		return -1;
 	}
+
 	return 0;
 }
 
